#!/bin/sh
PackagePath="/var/packages/MailServer"
#config template
PostfixTemplate="$PackagePath/target/etc/template/main.template"
DevecotTemplate="$PackagePath/target/etc/template/dovecot.template"
AliasTemplate="$PackagePath/target/etc/template/aliases.template"
MasterTemplate="$PackagePath/target/etc/template/master.template"
#smtp
MailServerConf="$PackagePath/etc/mailserver.conf"
PostfixConf="/var/packages/MailServer/target/etc/main.cf"
SetRelayAuth=$PackagePath/target/bin/syno_set_relay_auth
LoginMap="/var/packages/MailServer/target/etc/login_map"
GenConfValue=$PackagePath/target/bin/syno_gen_conf_value
PAM_UNIX_SRC="$PackagePath/target/etc/pam.d/pam.unix"
PAM_UNIX_BLOCK_SRC="$PackagePath/target/etc/pam.d/pam_block.unix"
PAM_WIN_SRC="$PackagePath/target/etc/pam.d/pam.win"
PAM_WIN_BLOCK_SRC="$PackagePath/target/etc/pam.d/pam_block.win"
PAM_LDAP_SRC="$PackagePath/target/etc/pam.d/pam.ldap"
PAM_LDAP_BLOCK_SRC="$PackagePath/target/etc/pam.d/pam_block.ldap"
PAM_SMTP_LINK="/etc/pam.d/smtp"
PAM_DOVECOT_LINK="/etc/pam.d/dovecot"

#alias
AliasDB="$PackagePath/etc/alias.db"
Table="alias_info_table"
AliasConf="/var/packages/MailServer/target/etc/aliases" #need modify after package location done
GroupTable="/etc/group"
PostAlias=/var/packages/MailServer/target/sbin/postalias
#protocol
DovecotConf="/var/packages/MailServer/target/etc/dovecot.conf"

#spam filter
MasterConf="/var/packages/MailServer/target/etc/master.cf"

# save all user string
ALL_USERS=""

# get ipv4only
IPV4=`/bin/get_key_value /etc.defaults/synoinfo.conf ipv4only`

# get account type
AccountType=`/bin/get_key_value $MailServerConf account_type`

# get DSM version
DSM_Major=`/bin/get_key_value /etc.defaults/VERSION majorversion`
DSM_Minor=`/bin/get_key_value /etc.defaults/VERSION minorversion`

# check DSM is support auto block PAM
HaveAutoBlockPam()
{
	if [ "$DSM_Major" -lt 4 ]; then
		return 0;
	elif [ "$DSM_Major" -eq 4 ] && [ "$DSM_Minor" -lt 2 ]; then
		return 0;
	fi
	return 1;
}

CreatePamLink()
{
	rm -rf ${PAM_SMTP_LINK}
	rm -rf ${PAM_DOVECOT_LINK}
	if [ "$AccountType" == "ldap" ]; then
		ln -sf ${PAM_LDAP_SRC} ${PAM_SMTP_LINK}
		ln -sf ${PAM_LDAP_BLOCK_SRC} ${PAM_DOVECOT_LINK}
	elif [ "$AccountType" == "win" ]; then
		ln -sf ${PAM_WIN_SRC} ${PAM_SMTP_LINK}
		ln -sf ${PAM_WIN_BLOCK_SRC} ${PAM_DOVECOT_LINK}
	else
		HaveAutoBlockPam
		if [ $? -eq "1" ]; then
			ln -sf ${PAM_UNIX_SRC} ${PAM_SMTP_LINK}
			ln -sf ${PAM_UNIX_BLOCK_SRC} ${PAM_DOVECOT_LINK}
		else
			ln -sf ${PAM_UNIX_SRC} ${PAM_SMTP_LINK}
			ln -sf ${PAM_UNIX_SRC} ${PAM_DOVECOT_LINK}
		fi
	fi
}

GenAliases() {
	$GenConfValue aliases
	$PostAlias $AliasConf
}

GenLdapAlias() {
	if [ "$AccountType" == "ldap" ]; then
		$GenConfValue ldap_alias
	fi
}

SMTP() {
	# create dovecot link for pam.d login
	CreatePamLink

	#gen aliases for Domain user
	GenAliases
	GenLdapAlias

	cat $PostfixTemplate > /tmp/tempinfo

	#smtp setting
	AliasMap="hash:/var/packages/MailServer/target/etc/aliases"
	if [ "$AccountType" == "ldap" ]; then
                AliasMap=${AliasMap}", ldap:/var/packages/MailServer/target/etc/ldap-aliases.cf"
        fi
	echo "alias_maps = $AliasMap" >> /tmp/tempinfo

	HostName=`/bin/get_key_value $MailServerConf smtp_hostname`
	if [ -z "$HostName" ]; then
		HostName="localhost"
	fi
	echo "myhostname = $HostName" >> /tmp/tempinfo

	SmtpAuth=`/bin/get_key_value $MailServerConf smtp_auth_enabled`
	if [ "$SmtpAuth" != "yes" ]; then
		SmtpAuth="no"
	fi
	echo "smtpd_sasl_auth_enable = $SmtpAuth" >> /tmp/tempinfo

	SmtpIgnoreLAN=`/bin/get_key_value $MailServerConf smtp_ignore_lan_auth`
	if [ "$SmtpAuth" = "yes" ] && [ "${SmtpIgnoreLAN}" = "no" ]; then
		if [ "$IPV4" = "yes" ]; then
			echo "mynetworks = 127.0.0.0/8" >> /tmp/tempinfo
		else
			echo "mynetworks = 127.0.0.0/8, [::1]/128" >> /tmp/tempinfo
		fi
	fi

	Domain=`/bin/get_key_value $MailServerConf canonical_domain`
	if [ -z  "$Domain" ]; then
		echo "mydestination = \$myhostname, localhost.\$mydomain, localhost" >> /tmp/tempinfo
	else
		echo "mydestination = \$myhostname, localhost.\$mydomain, localhost, $Domain" >> /tmp/tempinfo
	fi

	MessageSizeLimit=`/bin/get_key_value $MailServerConf message_limit`
	if [ -z "$MessageSizeLimit" ]; then
		MessageSizeLimit=10240000
	else
		MessageSizeLimit=`/usr/bin/expr $MessageSizeLimit \* 1048576`
	fi
	echo "message_size_limit = $MessageSizeLimit" >> /tmp/tempinfo

	RestrictSender=`/bin/get_key_value ${MailServerConf} restrict_sender`
	if [ ${RestrictSender} = "yes" ]; then
		echo "smtpd_sender_login_maps = hash:${LoginMap}" >> /tmp/tempinfo
		echo "smtpd_sender_restrictions = reject_authenticated_sender_login_mismatch, reject_unknown_sender_domain" >> /tmp/tempinfo
	else
		echo "smtpd_sender_restrictions = reject_unknown_sender_domain" >> /tmp/tempinfo
	fi

	SmtpEnable=`/bin/get_key_value $MailServerConf smtp_enabled`

	# SMTP-SSL
	SmtpsEnable=`/bin/get_key_value $MailServerConf smtp_ssl_enabled`
	if [ "$SmtpEnable" = "yes" ] && [ "$SmtpsEnable" = "yes" ]; then
		echo "smtpd_use_tls = yes" >> /tmp/tempinfo
		echo "tls_random_source = dev:/dev/urandom" >> /tmp/tempinfo
	fi

	#relay setting
	RelayEnable=`/bin/get_key_value $MailServerConf smtp_relay_enabled`
	if [ "$SmtpEnable" = "yes" ] && [ "$RelayEnable" = "yes" ]; then
		RelayServer=`/bin/get_key_value $MailServerConf smtp_relay_server`
		RelayPort=`/bin/get_key_value $MailServerConf smtp_relay_server_port`

		if [ ! -z "$RelayServer" ] && [ ! -z "$RelayPort" ]; then
			echo "relayhost = ${RelayServer}:${RelayPort}" >> /tmp/tempinfo
			RelayAuth=`/bin/get_key_value $MailServerConf smtp_relay_auth_enabled`
			RelayTLS=`/bin/get_key_value $MailServerConf smtp_relay_secure_conn`
			if [ "$RelayTLS" == "yes" ]; then
				echo "smtp_use_tls = $RelayTLS" >> /tmp/tempinfo
				echo "smtp_sasl_tls_security_options = noanonymous" >> /tmp/tempinfo
			fi

			if [ "$RelayAuth" == "yes" ]; then
				echo "smtp_sasl_auth_enable = $RelayAuth" >> /tmp/tempinfo
				echo "smtp_sasl_password_maps = hash:/var/packages/MailServer/target/etc/sasl_passwd" >> /tmp/tempinfo
				echo "smtp_sasl_security_options = noanonymous" >> /tmp/tempinfo
				$SetRelayAuth
			fi
		fi
	fi

	#set spam-filter: DNSBL
        SmtpdRecipientRestriction="permit_mynetworks, permit_sasl_authenticated, reject_unauth_destination"
	DNSBLEnable=`/bin/get_key_value $MailServerConf spam_dnsbl_enable`
	if [ "$DNSBLEnable" == "yes" ]; then
		DNSBLServer=`$GenConfValue dnsbl`
		if [ ! -z "${DNSBLServer}" ]; then
			SmtpdRecipientRestriction="${SmtpdRecipientRestriction}, ${DNSBLServer}"
		fi
	fi
	echo "smtpd_recipient_restrictions = ${SmtpdRecipientRestriction}" >> /tmp/tempinfo

	/bin/mv /tmp/tempinfo $PostfixConf

}

Alias() {
	GenAliases
}

Protocol() {

	cat $DevecotTemplate > /tmp/tempinfo
	for key in imap imaps pop3 pop3s; do
		Enabled=`/bin/get_key_value $MailServerConf ${key}_enabled`
		if [ "$Enabled" = "yes" ]; then
			Protocols="${Protocols} ${key}"
			if [ "${key}" = "imap" ]; then
				Imap="imap"
			fi
		fi
	done

	Imaps=`echo ${Protocols} | grep imaps`
	Pop3Pop3s=`echo ${Protocols} | grep pop3`

	if [ "$IPV4" = "yes" ]; then
		ListenValue="*"
	else
		ListenValue="*, [::]"
	fi

	if [ -z "$Imap" ]; then
		Protocols="imap${Protocols}"
		ListenValue="localhost"
	fi

	echo "protocols = $Protocols" >> /tmp/tempinfo

	echo "protocol imap {" >> /tmp/tempinfo
	echo "     listen = $ListenValue" >> /tmp/tempinfo
	if [ "$Imaps" ]; then
		if [ "$IPV4" = "yes" ]; then
			echo "     ssl_listen = *" >> /tmp/tempinfo
		else
			echo "     ssl_listen = *, [::]" >> /tmp/tempinfo
		fi
	fi
	echo "}" >> /tmp/tempinfo

	if [ "$Pop3Pop3s" ]; then
		echo "protocol pop3 {" >> /tmp/tempinfo
		if [ "$IPV4" = "yes" ]; then
			echo "     listen = *" >> /tmp/tempinfo
			echo "     ssl_listen = *" >> /tmp/tempinfo
		else
			echo "     listen = *, [::]" >> /tmp/tempinfo
			echo "     ssl_listen = *, [::]" >> /tmp/tempinfo
		fi

		echo "}" >> /tmp/tempinfo
	fi

	/bin/mv /tmp/tempinfo $DovecotConf
}

MasterCf() {
	cat $MasterTemplate > /tmp/tempinfo

	# SMTP-SSL
	local SmtpEnable=`/bin/get_key_value $MailServerConf smtp_enabled`
	local SmtpsEnable=`/bin/get_key_value $MailServerConf smtp_ssl_enabled`
	if [ "$SmtpEnable" = "yes" ] && [ "$SmtpsEnable" = "yes" ]; then
		echo "smtps inet n - n - - smtpd -o smtpd_tls_wrappermode=yes -o smtpd_sasl_auth_enable=yes" >> /tmp/tempinfo
	fi

	SpamEnable=`/bin/get_key_value $MailServerConf spam_enable`
	if [ "$SpamEnable" = "yes" ]; then
		echo "smtp inet n - n - - smtpd -o content_filter=spamassassin" >> /tmp/tempinfo
		echo "spamassassin unix - n n - - pipe user=spamfilter argv=/var/packages/MailServer/target/bin/filter.sh -oi -f \${sender} \${recipient}" >> /tmp/tempinfo
	else
		echo "smtp inet n - n - - smtpd" >> /tmp/tempinfo
	fi
	/bin/mv /tmp/tempinfo $MasterConf

}

GetAllUsers() {
    synouser --enum local | sed '1d' | while read user
    do
        ALL_USERS=${ALL_USERS}","${user}
        echo ${ALL_USERS} > /tmp/user.all
    done
    ALL_USERS=`cat /tmp/user.all | sed 's/.\(.*\)/\1/'`
    rm -rf /tmp/user.all
}

case "$1" in
	smtp)
		SMTP
		MasterCf
		;;
	protocol)
		Protocol
		;;
	alias)
		Alias
		;;
    spam)
		MasterCf
		;;
	*)
		echo "Usage: $0 {smtp|protocol|alias|spam}" >&2
		exit 1
		;;
esac
